import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { BaseDbService } from '../commons/services/base-db.service';
import { CreateArticleInput } from './dto/create-article.input';
import { UpdateArticleInput } from './dto/update-article.input';
import { Article } from './entities/article.entity';

@Injectable()
export class ArticlesService extends BaseDbService<Article> {
  readonly uniqueFields: Array<keyof Article> = ['title'];
  constructor(
    @InjectRepository(Article)
    readonly repository: Repository<Article>,
  ) {
    super(repository);
  }
  async create(createArticleInput: CreateArticleInput) {
    await this.isDuplicateEntity(createArticleInput);
    return await this.repository.save(
      this.repository.create(createArticleInput),
    );
  }

  async findAll() {
    return await this.repository.find({
      order: { createdAt: 'DESC' },
    });
  }

  async update(article: Article, updateArticleInput: UpdateArticleInput) {
    await this.isDuplicateEntityForUpdate(article.id, updateArticleInput);
    return await this.repository.save(
      this.repository.merge(article, updateArticleInput),
    );
  }

  async remove(id: string) {
    await this.canRemove([id]);
    return await this.repository.softDelete({ id }).then((d) => d.affected);
  }
}
